package data

/**
 * Schrodinger is an analogue to Scala's Option and a special case of
 * Superposition where a cat may or may not exist and the cat must be specially
 * handled as if it were simultaneously alive or dead.
 */
sealed trait Schrodinger[+A] {

  def cat: A
}

case class Alive[+A](cat: A) extends Schrodinger[A]

case object Dead extends Schrodinger[Nothing] {

  def cat: Nothing = throw new Exception("He's dead Jim!")
}
