package typeclasses

import data.Superposition

/**
 * Defines the MonadOperations operations for any type of kind F[_] for error
 * type E. MonadError is a special case of ApplicativeError and Monad that
 * further augments error handling operations.
 */
trait MonadError[F[_], E] extends Monad[F] with ApplicativeError[F, E] {

  /**
   * Replaces A with E if the lifted A does not satisfy the predicate.
   */
  def ensure[A](fa: F[A])(error: => E)(predicate: A => Boolean): F[A]

  /**
   * Replaces A with E by means of A if the lifted A does not satisfy the predicate.
   */
  def ensureOr[A](fa: F[A])(error: A => E)(predicate: A => Boolean): F[A]

  /**
   * Applies a partial function to the E, if any.
   */
  def adaptError[A](fa: F[A])(pf: PartialFunction[E, E]): F[A]

  /**
   * Inverse of ApplicativeError attempt().
   */
  def rethrow[A, EE <: E](fa: F[Superposition[EE, A]]): F[A]
}
